@@ -1,5 +1,11 @@
+"""
+The ``aimlflow.AiService`` module provides an API for accessing ai service objects.
+An AiService object is equivalent to an project and is required to create and monitor ml worklows.
+"""
+
 import os.path
 from os import path
+from os import environ
 
 import json
 import time
@@ -12,8 +18,9 @@ import aimlflow.utils as utils
 # Swami
 #BASE_URL = 'https://xh6svzuqii.execute-api.us-east-1.amazonaws.com/Navigator'
 # Dev
-BASE_URL = 'https://ygermtztdb.execute-api.us-east-1.amazonaws.com/Navigator'
-
+# BASE_URL = 'https://ygermtztdb.execute-api.us-east-1.amazonaws.com/Navigator'
+# PROD
+BASE_URL = 'https://b3005vfmhf.execute-api.us-east-1.amazonaws.com/Navigator'
 
 class AiService:
     _access_token = None
@@ -49,7 +56,7 @@ class AiService:
         return True
 
 
-    def get_user_info(self):
+    def _get_user_info(self):
         url = BASE_URL + '/get-user-info'
         try:
             params = {'AccessToken':  self._access_token}
@@ -64,8 +71,45 @@ class AiService:
         return None
 
 
-    def auth(self, username, password):
+    def auth(self, username=None, password=None):
+        """
+        Authenticate a user with Navigator.
+        :param username: AIClub/Navigator username. You can also set username via the OS environment variable.
+        :param password: AIClub/Navigator username. You can also set password via the OS environment variable.
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            import os
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+
+            os.environ['username'] = '<your username>'
+            os.environ['password'] = '<your password>'
+            response = aiservice.auth()
+            if response == True:
+                print('Successfully Authenticated!')
+
+        .. code-block:: python
+            :caption: Example using function parameters (Alternative to environment variables)
+
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+
+            response = aiservice.auth(username='<your username>', password='<your password>')
+            if response == True:
+                print('Successfully Authenticated!')
+        """
         result = False
+        if not username:
+            #check for os environment to get the username and password
+            try:
+                username = environ['username']
+                password = environ['password']
+            except:
+                utils.color_print('Please provide username either via params or os.environ!')
+                return False
         try:
             url = BASE_URL + '/get-token'
             params = {'username': username, 'password': password}
@@ -76,14 +120,14 @@ class AiService:
             try:
                 response_json = response.json()
             except Exception as e:
-                print('User Authentication failed!')
+                utils.color_print('User Authentication failed!', 'RED')
                 return False
 
             self._id_token = response_json['AuthenticationResult']['IdToken']
             self._access_token = response_json['AuthenticationResult']['AccessToken']
 
             if utils.check_email(username):
-                response = self.get_user_info()
+                response = self._get_user_info()
                 if not response:
                     utils.color_print('Unable to get user information for authentication!' \
                         'Please contact support@pyxeda.ai for assistance.', 'RED')
@@ -103,6 +147,21 @@ class AiService:
 
 
     def createService(self, name):
+        """
+        Create a new AI Service.
+        :param name: Name of the service.
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            # Assuming already authenticated
+
+            response = aiservice.CreateService(name='my_ai_service')
+            print('service: ', response)
+
+        """
         is_auth = self._check_authentication()
         if not is_auth:
             return False
@@ -134,6 +193,21 @@ class AiService:
 
 
     def getService(self, name):
+        """
+        Fetch an existing AI Service.
+        :param name: Name of the service.
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            # Assuming already authenticated
+
+            response = aiservice.getService(name='my_ai_service')
+            print('service: ', response)
+
+        """
         is_auth = self._check_authentication()
         if not is_auth:
             return None
@@ -186,11 +260,11 @@ class AiService:
                 utils.color_print('Successfully Uploaded dataset!', 'GREEN')
                 response = response_json
         except Exception as e:
-            utils.color_print('Unable to upload dataset! Error: '.format(str(e)), 'RED')
+            utils.color_print('Unable to upload dataset! Error: {}'.format(str(e)), 'RED')
             response = None
         return response
 
-    def uploadData(self, params=None):
+    def uploadData(self, **kwargs):
         is_auth = self._check_authentication()
         if not is_auth:
             return None
@@ -198,26 +272,25 @@ class AiService:
         response = None
         try:
             file_name = None
-            if params:
-                if 'fileName' in params:
-                    file_name = params['fileName']
-                if 'cloud' in params:
-                    cloud = params['cloud']
+            if 'fileName' in kwargs:
+                file_name = kwargs['fileName']
+                if 'cloud' in kwargs:
+                    cloud = kwargs['cloud']
                     if cloud != 'AWS':
                         utils.color_print('{} is not yet supported!'.format(cloud), 'RED')
                         return None
-                else:
-                    cloud = 'AWS'
-                if cloud == 'AWS':
-                    url = BASE_URL + '/storage/s3/uploadLink'
-                if not 'raw_data' in params and 'localFile' in params and not params['localFile']:
-                    print('raw_data or params[\"localFile\"] is required to uploadData')
-                    return None
+            else:
+                cloud = 'AWS'
+            if cloud == 'AWS':
+                url = BASE_URL + '/storage/s3/uploadLink'
+            if not 'raw_data' in kwargs and 'localFile' in kwargs and not kwargs['localFile']:
+                print('raw_data or params[\"localFile\"] is required to uploadData')
+                return None
 
             # TODO: handle linked accounts
             upload_params = {
                 'bucket': self._username,
-                'key': params['fileName'],
+                'key': kwargs['fileName'],
                 'method': 'PUT',
                 'contentType': 'text/csv',
             }
@@ -228,10 +301,10 @@ class AiService:
             upload_url = response_json['url']
 
             headers = {'Content-Type': 'text/csv'}
-            if 'rawData' in params and params['rawData']:
-                fileobj = params['rawData']
+            if 'rawData' in kwargs and kwargs['rawData']:
+                fileobj = kwargs['rawData']
             else:
-                fileobj = open(params['localFile'], 'rb')
+                fileobj = open(kwargs['localFile'], 'rb')
 
             # Upload the file to the cloud location
             response = requests.put(upload_url, headers=headers, data=fileobj)
@@ -240,7 +313,7 @@ class AiService:
             # Import the data and get a data id
             import_params = {
                 'serviceName': self._service_name,
-                'location': {'bucket': self._username, 'key': params['fileName']},
+                'location': {'bucket': self._username, 'key': kwargs['fileName']},
                 'dataType': 'csv',
                 'cloud': 'AWS',
                 'source': 's3',
@@ -255,48 +328,125 @@ class AiService:
         return response
 
 
-    def uploadLocalData(self, params):
-        if not 'localFile' in params or not params['localFile']:
-            utils.color_print('Need the complete path of the file in params[\"localFile\"]!', 'RED')
+    def uploadLocalData(self, **kwargs):
+        """
+        Upload a file from your local environment to the cloud.
+        :param localFile: Location of the local file (include the complete path).
+        :param fileName: Destination file name in the cloud.
+        :param cloud: Cloud to use. Current, only AWS is supported
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            ...
+
+            response = aiservice.uploadLocalData(localFile='<complete path to local file>',
+                                                 fileName='<cloud file name>',
+                                                 cloud='AWS')
+            print('uploadLocalData: ', response)
+
+        """
+        if not 'localFile' in kwargs or not kwargs['localFile']:
+            utils.color_print('Need the complete path of the file in input parameters [\"localFile\"]!', 'RED')
             return None
-        if not path.exists(params['localFile']):
-            utils.color_print('LocalFile {} does not exist!'.format(params['localFile']), 'RED')
+        if not path.exists(kwargs['localFile']):
+            utils.color_print('LocalFile {} does not exist!'.format(kwargs['localFile']), 'RED')
             return None
         # check to see if you have a local file params and its a valid file
-        return self.uploadData(params)
+        return self.uploadData(**kwargs)
 
-    def uploadRawData(self, params):
-        if not 'rawData' in params or not params['rawData']:
-            utils.color_print('Need the raw data in bytes in params[\"rawData\"]!', 'RED')
+    def uploadRawData(self, **kwargs):
+        """
+        Upload a data in the form of a byte string from your local environment to the cloud.
+        :param rawData: Data in bytes.
+        :param fileName: Destination file name in the cloud.
+        :param cloud: Cloud to use. Current, only AWS is supported
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            ...
+
+            response = aiservice.uploadRawData(rawRdata='<raw data in bytes>',
+                                                 fileName='<cloud file name>',
+                                                 cloud='AWS')
+            print('uploadRawlData: ', response)
+
+        """
+        if not 'rawData' in kwargs or not kwargs['rawData']:
+            utils.color_print('Need the raw data in bytes in the input arguments [\"rawData\"]!', 'RED')
             return None
-        if not isinstance(params['rawData'], bytes):
+        if not isinstance(kwargs['rawData'], bytes):
             utils.color_print('Raw data of the file should be in bytes!')
             return None
-        return self.uploadData(params)
+        return self.uploadData(**kwargs)
 
 
-    def uploadJSONData(self, params):
-        if not 'jsonData' in params or not params['jsonData']:
+    def uploadJSONData(self, **kwargs):
+        """
+        Upload a data in the form of a JSON from your local environment to the cloud.
+        :param jsonData: Data in JSON format.
+        :param fileName: Destination file name in the cloud.
+        :param cloud: Cloud to use. Current, only AWS is supported
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            ...
+
+            response = aiservice.uploadJSONData(jsonRdata=<JSON data>,
+                                                 fileName='<cloud file name>',
+                                                 cloud='AWS')
+            print('uploadRawlData: ', response)
+
+        """
+        if not 'jsonData' in kwargs or not kwargs['jsonData']:
             utils.color_print('Need the data as a json string in params[\"jsonData\"]!', 'RED')
             return None
         try:
-            tmp_json = json.loads(params['jsonData'])
+            tmp_json = json.loads(kwargs['jsonData'])
             del tmp_json
         except Exception as e:
             utils.color_print('Need the data to be a valid json string in params[\"jsonData\"]!', 'RED')
         try:
-            read_file = pd.read_json(params['jsonData'])
+            read_file = pd.read_json(kwargs['jsonData'])
             csv_string = read_file.to_csv(index=False)
             csv_bytes = csv_string.encode()
-            del params['jsonData']
-            params['rawData'] = csv_bytes
-            return self.uploadRawData(params)
+            del kwargs['jsonData']
+            kwargs['rawData'] = csv_bytes
+            return self.uploadRawData(**kwargs)
         except Exception as e:
             utils.color_print('Unable to upload jsonData!', 'RED')
             return None
 
     #TODO: Add a parameter to decide on the wait time
     def getFEStatus(self, fe_id=None, blocking=False, wait_time=None):
+        """
+        Fetch the status of the Feature Engineering experiment.
+        :param fe_id: Feature Enginering Identifier (returned from launchFE).
+        :param blocking: Boolean value indicating if the call should wait until completion or failure.
+                         Default is False.
+        :param wait_time: Wait time in seconds between subsequent status checks. Default is 30 seconds
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            ...
+
+            response = aiservice.getFEStatus(fe_id=<Feature engineering ID>,
+                                                 blocking=True,
+                                                 wait_time=30)
+            print('FE Status: ', response['status'])
+            print('FE Details: ', response)
+        """
         is_auth = self._check_authentication()
         if not is_auth:
             return None
@@ -342,6 +492,29 @@ class AiService:
 
 
     def launchFE(self, params):
+        """
+        Launch a Feature Engineering experiment.
+        :param dataSourceId: Data Source Identifier (returned from uploadData).
+        :param column: Label column name of the dataset.
+        :param waitTime: Wait time in seconds between subsequent status checks. Default is 30 seconds.
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            ...
+
+            fe_params = {
+                'column' = '<label column>'
+                'waitTime' = 60
+            }
+            response = aiservice.launchFE(dataSourceId=<Data Source ID>,
+                                          blocking=True,
+                                          waitTime=30)
+            print('FE Status: ', response['status'])
+            print('FE Details: ', response)
+        """
         is_auth = self._check_authentication()
         if not is_auth:
             return None
@@ -434,6 +607,26 @@ class AiService:
 
 
     def getTrainingStatus(self, train_id, blocking=False, wait_time=None):
+        """
+        Fetch the status of the Training experiment.
+        :param train_id: Training Experiment Identifier (returned from launchTrain).
+        :param blocking: Boolean value indicating if the call should wait until completion or failure.
+                         Default is False.
+        :param wait_time: Wait time in seconds between subsequent status checks. Default is 30 seconds
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            ...
+
+            response = aiservice.getTrainStatus(train_id=<Train Experiment ID>,
+                                                blocking=True,
+                                                wait_time=30)
+            print('Train Status: ', response['status'])
+            print('Train Details: ', response)
+        """
         is_auth = self._check_authentication()
         if not is_auth:
             return None
@@ -493,6 +686,30 @@ class AiService:
 
 
     def launchTrain(self, params):
+        """
+        Launch a Training experiment.
+        :param fe_id: Feature Engineering Identifier (returned from launchFE).
+        :param launchMode: The mode to run the train experiment. In the current version, only 'automatic' is supported.
+        :param waitTime: Wait time in seconds between subsequent status checks. Default is 30 seconds.
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            ...
+
+            fe_params = {
+                'column' = '<label column>'
+                'waitTime' = 60
+            }
+            response = aiservice.launchTrain(fe_id=<Feature engineering ID>,
+                                             launchMode='automatic',
+                                             engine='aws-sklearn-serverless',
+                                             waitTime=30)
+            print('Train Status: ', response['status'])
+            print('Train Details: ', response)
+        """
         is_auth = self._check_authentication()
         if not is_auth:
             return None
@@ -572,6 +789,21 @@ class AiService:
             return None
 
     def getEndpointURL(self, service_name=None):
+        """
+        Get the endpoint (or model serving) URL of the AI Service
+        :param service_name: AI Service Name. Will use the current Ai Service name if none is provided.
+
+
+        .. code-block:: python
+            :caption: Example using environment variables
+            from aimlflow import AiService
+
+            aiservice = AiService.AiService()
+            ...
+
+            response = aiservice.getEndpointURL()
+            print('Endpoint URL: ', response)
+        """
         is_auth = self._check_authentication()
         if not is_auth:
             return None
